@@ -4,18 +4,13 @@ Runs a model on a single node across multiple gpus.
 import os
 from argparse import ArgumentParser
 
-import numpy as np
-import torch
-
-import pytorch_lightning as pl
+from pytorch_lightning import Trainer, seed_everything
 from pl_examples.models.lightning_template import LightningTemplateModel
 
-SEED = 2334
-torch.manual_seed(SEED)
-np.random.seed(SEED)
+seed_everything(234)
 
 
-def main(hparams):
+def main(args):
     """
     Main training routine specific for this project
     :param hparams:
@@ -23,17 +18,12 @@ def main(hparams):
     # ------------------------
     # 1 INIT LIGHTNING MODEL
     # ------------------------
-    model = LightningTemplateModel(**vars(hparams))
+    model = LightningTemplateModel(**vars(args))
 
     # ------------------------
     # 2 INIT TRAINER
     # ------------------------
-    trainer = pl.Trainer(
-        max_epochs=hparams.epochs,
-        gpus=hparams.gpus,
-        distributed_backend=hparams.distributed_backend,
-        precision=16 if hparams.use_16bit else 32,
-    )
+    trainer = Trainer.from_argparse_args(args)
 
     # ------------------------
     # 3 START TRAINING
@@ -41,7 +31,7 @@ def main(hparams):
     trainer.fit(model)
 
 
-if __name__ == '__main__':
+def run_cli():
     # ------------------------
     # TRAINING ARGUMENTS
     # ------------------------
@@ -50,31 +40,16 @@ if __name__ == '__main__':
     root_dir = os.path.dirname(os.path.realpath(__file__))
     parent_parser = ArgumentParser(add_help=False)
 
-    # gpu args
-    parent_parser.add_argument(
-        '--gpus',
-        type=int,
-        default=2,
-        help='how many gpus'
-    )
-    parent_parser.add_argument(
-        '--distributed_backend',
-        type=str,
-        default='dp',
-        help='supports four options dp, ddp, ddp2, ddp_spawn'
-    )
-    parent_parser.add_argument(
-        '--use_16bit',
-        dest='use_16bit',
-        action='store_true',
-        help='if true uses 16 bit precision'
-    )
-
     # each LightningModule defines arguments relevant to it
     parser = LightningTemplateModel.add_model_specific_args(parent_parser, root_dir)
-    hyperparams = parser.parse_args()
+    parser = Trainer.add_argparse_args(parser)
+    args = parser.parse_args()
 
     # ---------------------
     # RUN TRAINING
     # ---------------------
-    main(hyperparams)
+    main(args)
+
+
+if __name__ == '__main__':
+    run_cli()
