@@ -190,22 +190,41 @@ class Agent:
 class DQNLightning(pl.LightningModule):
     """ Basic DQN Model """
 
-    def __init__(self, hparams: argparse.Namespace) -> None:
+    def __init__(self,
+                 replay_size,
+                 warm_start_steps: int,
+                 gamma: float,
+                 eps_start: int,
+                 eps_end: int,
+                 eps_last_frame: int,
+                 sync_rate,
+                 lr: float,
+                 episode_length,
+                 batch_size, **kwargs) -> None:
         super().__init__()
-        self.hparams = hparams
+        self.replay_size = replay_size
+        self.warm_start_steps = warm_start_steps
+        self.gamma = gamma
+        self.eps_start = eps_start
+        self.eps_end = eps_end
+        self.eps_last_frame = eps_last_frame
+        self.sync_rate = sync_rate
+        self.lr = lr
+        self.episode_length = episode_length
+        self.batch_size = batch_size
 
-        self.env = gym.make(self.hparams.env)
+        self.env = gym.make(self.env)
         obs_size = self.env.observation_space.shape[0]
         n_actions = self.env.action_space.n
 
         self.net = DQN(obs_size, n_actions)
         self.target_net = DQN(obs_size, n_actions)
 
-        self.buffer = ReplayBuffer(self.hparams.replay_size)
+        self.buffer = ReplayBuffer(self.replay_size)
         self.agent = Agent(self.env, self.buffer)
         self.total_reward = 0
         self.episode_reward = 0
-        self.populate(self.hparams.warm_start_steps)
+        self.populate(self.warm_start_steps)
 
     def populate(self, steps: int = 1000) -> None:
         """
@@ -250,7 +269,7 @@ class DQNLightning(pl.LightningModule):
             next_state_values[dones] = 0.0
             next_state_values = next_state_values.detach()
 
-        expected_state_action_values = next_state_values * self.hparams.gamma + rewards
+        expected_state_action_values = next_state_values * self.gamma + rewards
 
         return nn.MSELoss()(state_action_values, expected_state_action_values)
 
@@ -267,8 +286,8 @@ class DQNLightning(pl.LightningModule):
             Training loss and log metrics
         """
         device = self.get_device(batch)
-        epsilon = max(self.hparams.eps_end, self.hparams.eps_start -
-                      self.global_step + 1 / self.hparams.eps_last_frame)
+        epsilon = max(self.eps_end, self.eps_start -
+                      self.global_step + 1 / self.eps_last_frame)
 
         # step through environment with agent
         reward, done = self.agent.play_step(self.net, epsilon, device)
@@ -282,7 +301,7 @@ class DQNLightning(pl.LightningModule):
             self.episode_reward = 0
 
         # Soft update of target network
-        if self.global_step % self.hparams.sync_rate == 0:
+        if self.global_step % self.sync_rate == 0:
             self.target_net.load_state_dict(self.net.state_dict())
 
         log = {'total_reward': torch.tensor(self.total_reward).to(device),
@@ -293,16 +312,17 @@ class DQNLightning(pl.LightningModule):
 
     def configure_optimizers(self) -> List[Optimizer]:
         """Initialize Adam optimizer"""
-        optimizer = optim.Adam(self.net.parameters(), lr=self.hparams.lr)
+        optimizer = optim.Adam(self.net.parameters(), lr=self.lr)
         return [optimizer]
 
     def __dataloader(self) -> DataLoader:
         """Initialize the Replay Buffer dataset used for retrieving experiences"""
-        dataset = RLDataset(self.buffer, self.hparams.episode_length)
-        dataloader = DataLoader(dataset=dataset,
-                                batch_size=self.hparams.batch_size,
-                                sampler=None
-                                )
+        dataset = RLDataset(self.buffer, self.episode_length)
+        dataloader = DataLoader(
+            dataset=dataset,
+            batch_size=self.batch_size,
+            sampler=None,
+        )
         return dataloader
 
     def train_dataloader(self) -> DataLoader:
@@ -314,8 +334,8 @@ class DQNLightning(pl.LightningModule):
         return batch[0].device.index if self.on_gpu else 'cpu'
 
 
-def main(hparams) -> None:
-    model = DQNLightning(hparams)
+def main(args) -> None:
+    model = DQNLightning(**vars(args))
 
     trainer = pl.Trainer(
         gpus=1,
