@@ -14,7 +14,7 @@ from torch.utils.tensorboard import SummaryWriter
 
 from pytorch_lightning import _logger as log
 from pytorch_lightning.core.saving import save_hparams_to_yaml
-from pytorch_lightning.loggers.base import LightningLoggerBase
+from pytorch_lightning.loggers.base import LightningLoggerBase, rank_zero_experiment
 from pytorch_lightning.utilities import rank_zero_only
 
 
@@ -83,6 +83,7 @@ class TensorBoardLogger(LightningLoggerBase):
         return log_dir
 
     @property
+    @rank_zero_experiment
     def experiment(self) -> SummaryWriter:
         r"""
         Actual tensorboard object. To use TensorBoard features in your
@@ -96,6 +97,7 @@ class TensorBoardLogger(LightningLoggerBase):
         if self._experiment is not None:
             return self._experiment
 
+        assert rank_zero_only.rank == 0, 'tried to init log dirs in non global_rank=0'
         os.makedirs(self.root_dir, exist_ok=True)
         self._experiment = SummaryWriter(log_dir=self.log_dir, **self._kwargs)
         return self._experiment
@@ -135,6 +137,8 @@ class TensorBoardLogger(LightningLoggerBase):
 
     @rank_zero_only
     def log_metrics(self, metrics: Dict[str, float], step: Optional[int] = None) -> None:
+        assert rank_zero_only.rank == 0, 'experiment tried to log from global_rank != 0'
+
         for k, v in metrics.items():
             if isinstance(v, torch.Tensor):
                 v = v.item()
