@@ -127,6 +127,7 @@ import torch
 from pytorch_lightning import _logger as log
 from pytorch_lightning.callbacks import ModelCheckpoint
 from pytorch_lightning.loggers import LightningLoggerBase
+from pytorch_lightning.utilities import NATIVE_AMP_AVALAIBLE
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
 from pytorch_lightning.utilities.distributed import rank_zero_only, rank_zero_warn, rank_zero_info
 
@@ -139,7 +140,7 @@ else:
 
 try:
     import horovod.torch as hvd
-except ImportError:
+except (ModuleNotFoundError, ImportError):
     HOROVOD_AVAILABLE = False
 else:
     HOROVOD_AVAILABLE = True
@@ -171,13 +172,11 @@ class TrainerDDPMixin(ABC):
     num_gpu_nodes: int
     gpus: List[int]
     logger: Union[LightningLoggerBase, bool]
-    checkpoint_callback: Union[ModelCheckpoint, bool]
     data_parallel_device_ids: ...
     distributed_backend: Optional[str]
     amp_level: str
     use_tpu: bool
     default_root_dir: str
-    use_native_amp: bool
     progress_bar_callback: ...
     num_processes: int
     num_nodes: int
@@ -185,7 +184,8 @@ class TrainerDDPMixin(ABC):
     tpu_cores: int
 
     @property
-    def is_global_zero(self) -> int:
+    @abstractmethod
+    def is_global_zero(self) -> bool:
         """Warning: this is just empty shell for code implemented in other class."""
 
     @property
@@ -218,6 +218,14 @@ class TrainerDDPMixin(ABC):
     def save_checkpoint(self, *args):
         """Warning: this is just empty shell for code implemented in other class."""
 
+    @abstractmethod
+    def setup(self, *args) -> None:
+        """Warning: this is just empty shell for code implemented in other class."""
+
+    @abstractmethod
+    def is_function_implemented(self, *args) -> bool:
+        """Warning: this is just empty shell for code implemented in other class."""
+
     def init_tpu(self):
         # turn off all the GPU stuff
         self.distributed_backend = None
@@ -509,8 +517,8 @@ class TrainerDDPMixin(ABC):
 
         # AMP
         # run through amp wrapper before going to distributed DP
-        # TODO: remove in v0.8.0
-        if self.use_amp and not self.use_native_amp:
+        # TODO: remove with dropping NVIDIA AMP support
+        if self.use_amp and not NATIVE_AMP_AVALAIBLE:
             model, optimizers = model.configure_apex(amp, model, self.optimizers, self.amp_level)
             self.optimizers = optimizers
             self.reinit_scheduler_properties(self.optimizers, self.lr_schedulers)
