@@ -188,7 +188,6 @@ class TrainerTrainingTricksMixin(ABC):
             'callbacks': self.callbacks,
             'checkpoint_callback': self.checkpoint_callback,
             'early_stop_callback': self.early_stop_callback,
-            'enable_early_stop': self.enable_early_stop,
             'auto_scale_batch_size': self.auto_scale_batch_size,
             'limit_train_batches': self.limit_train_batches,
             'model': self.model,
@@ -202,7 +201,6 @@ class TrainerTrainingTricksMixin(ABC):
         self.callbacks = []  # not needed before full run
         self.checkpoint_callback = False  # required for saving
         self.early_stop_callback = None
-        self.enable_early_stop = False
         self.limit_train_batches = 1.0
         self.optimizers, self.schedulers = [], []  # required for saving
         self.model = model  # required for saving
@@ -215,7 +213,6 @@ class TrainerTrainingTricksMixin(ABC):
         self.checkpoint_callback = self.__dumped_params['checkpoint_callback']
         self.auto_scale_batch_size = self.__dumped_params['auto_scale_batch_size']
         self.early_stop_callback = self.__dumped_params['early_stop_callback']
-        self.enable_early_stop = self.__dumped_params['enable_early_stop']
         self.limit_train_batches = self.__dumped_params['limit_train_batches']
         self.model = self.__dumped_params['model']
         del self.__dumped_params
